//*******************************************************
//       Copyright (c) 2002-2003 Gena A. Petrov
//*******************************************************
unit PropEditors;

interface

uses Classes, StdCtrls, TypInfo, MacrosHandle, Controls, Windows;

type
  TPropCheckBox = class(TCheckBox)
  private
    FPostponeUpdate: Boolean;
    PR: TPropRec;
    FSetEnumName: String;
    FPropertyName: string;
    procedure SetSetEnumName(Value: string);
    procedure SetPropertyName(Value: string);
  protected
    procedure Loaded; override;
    procedure Toggle; override;
    procedure GetCompPropValue;
    procedure SetCompPropValue;
  public
    procedure Update; override;
    procedure UpdatePropertyValue;
  published
    property PostponeUpdate: Boolean read FPostponeUpdate write FPostponeUpdate;
    property SetEnumName: string read FSetEnumName write SetSetEnumName;
    property PropertyName: string read FPropertyName write SetPropertyName;
  end;

  TPropEdit = class(TEdit)
  private
    FPostponeUpdate: Boolean;
    PR: TPropRec;
    FPropertyName: string;
    procedure SetPropertyName(Value: string);
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoExit; override;
    procedure Loaded; override;
    procedure GetCompPropValue;
    procedure SetCompPropValue;
    procedure UpdatePropertyValue;
  published
    property PostponeUpdate: Boolean read FPostponeUpdate write FPostponeUpdate;
    property PropertyName: string read FPropertyName write SetPropertyName;
  end;

  TPropRadioButton = class(TRadioButton)
  private
    FInitialize: Boolean;
    FPostponeUpdate: Boolean;
    FEnumName: String;
    PR: TPropRec;
    FPropertyName: string;
    procedure SetPropertyName(Value: string);
    procedure SetEnumName(Value: string);
  protected
    procedure Loaded; override;
    procedure GetCompPropValue;
    procedure SetCompPropValue;
  public
    procedure SetChecked(Value: Boolean); override;
    procedure UpdatePropertyValue;
  published
    property PostponeUpdate: Boolean read FPostponeUpdate write FPostponeUpdate;
    property EnumName: string read FEnumName write SetEnumName;
    property PropertyName: string read FPropertyName write SetPropertyName;
  end;

procedure Register;

implementation

uses
  Consts,
  FCConsts,
  SysUtils;

procedure Register;
begin
  RegisterComponents('FormsConstructor', [TPropCheckBox, TPropEdit, TPropRadioButton]);
end;

// TPropCheckBox

procedure TPropCheckBox.Loaded;
begin
  inherited;
  GetCompPropValue;
end;

procedure TPropCheckBox.Toggle;
begin
  inherited;
  if (csDesigning in ComponentState) then exit;
  if FPostponeUpdate then exit;
  SetCompPropValue;
end;

procedure TPropCheckBox.Update;
begin
  inherited;
  GetCompPropValue;
end;

procedure TPropCheckBox.SetPropertyName(Value: string);
begin
  if Value = FPropertyName then exit;

  FPropertyName := Value;

  GetCompPropValue;
end;

procedure TPropCheckBox.SetSetEnumName(Value: string);
begin
  if Value = FSetEnumName then exit;

  FSetEnumName := Trim(Value);

  GetCompPropValue;
end;

procedure TPropCheckBox.GetCompPropValue;
var
  i: Integer;
  BaseType: PPTypeInfo;
  PV: TPropValue;
  z: Integer;
  S: String;
begin
  if (csLoading in ComponentState) or
   (csDesigning in ComponentState) then exit;

  Enabled := False;
  Checked:= False;

  if FPropertyName = '' then exit;

  try
    PV := MacrosHandler.GetPropValueByMacros(FPropertyName,PR);

    with PV do
    begin
      with (PPropInfo(PR.CommandPointer))^.PropType^^ do
      begin
        case Kind of
          tkEnumeration:
          begin
            if ( Name = 'Boolean' ) then
            begin
              Enabled := True;
              Checked:= Boolean(Addr^);
            end;
          end;
          tkSet:
          begin
            Enabled := True;
            BaseType := GetTypeData(PPropInfo(PR.CommandPointer)^.PropType^)^.CompType;
            Z:= Integer(PV.Addr^);
            for i := 0 to SizeOf(TIntegerSet) * 8 - 1 do
            begin
              if ( i in TIntegerSet(z) ) then
              begin
                S:= GetEnumName(BaseType^, i);
                if S = FSetEnumName then
                begin
                  Checked:= True;
                  break;
                end;
              end;
            end;  
          end;
        end;
      end;
    end;
  except
    MessageBox(Self.Handle, PChar(SCantGet), PChar(SError),mb_IconError);
  end;
end;

procedure TPropCheckBox.SetCompPropValue;
const
  BooleanStr : array[False..True] of String[5] = ('False','True');
  BooleanChar: array[False..True] of Char = ('-','+');
begin
  if FPropertyName = '' then exit;
  try
    with (PPropInfo(PR.CommandPointer))^.PropType^^ do
    begin
      case Kind of
        tkEnumeration:
          if ( Name = 'Boolean' ) then
            MacrosHandler.ExecMacros(FPropertyName + ':= '+BooleanStr[Checked]);
        tkSet:
          MacrosHandler.ExecMacros(FPropertyName + BooleanChar[Checked] + '= '+FSetEnumName);
      end;
    end;
  except
    MessageBox(Self.Handle, PChar(SCantSet),PChar(SError),mb_IconError);
  end;
end;

procedure TPropCheckBox.UpdatePropertyValue;
begin
  SetCompPropValue;
end;


// TPropEdit

procedure TPropEdit.Loaded;
begin
  inherited;
  GetCompPropValue;
end;

procedure TPropEdit.SetPropertyName(Value: string);
begin
  if Value = FPropertyName then exit;

  FPropertyName := Value;

  GetCompPropValue;
end;

procedure TPropEdit.GetCompPropValue;
var
  PV: TPropValue;
begin
  if (csLoading in ComponentState) or
   (csDesigning in ComponentState) then exit;

  Enabled := False;

  if FPropertyName = '' then exit;

  try
    PV := MacrosHandler.GetPropValueByMacros(FPropertyName,PR);

    with PV do
    begin
      with (PPropInfo(PR.CommandPointer))^.PropType^^ do
      begin
        case Kind of
          tkString, tkLString, tkWString:
          begin
            Enabled := True;
            Text:= String(Addr^)
          end;
          tkInteger:
          begin
            Enabled := True;
            Text:= IntToStr(Integer(Addr^));
          end;
        end;
      end;
    end;
  except
    MessageBox(Self.Handle, PChar(SCantGet),PChar(SError),mb_IconError);
  end;
end;

procedure TPropEdit.SetCompPropValue;
begin
  if FPropertyName = '' then exit;
  try
    with (PPropInfo(PR.CommandPointer))^.PropType^^ do
    begin
      case Kind of
        tkString, tkLString, tkWString:
          MacrosHandler.ExecMacros(FPropertyName + '= "' + Text + '"');
        tkInteger:
          MacrosHandler.ExecMacros(FPropertyName + '= ' + Text);
      end;
    end;
  except
    MessageBox(Self.Handle, PChar(SCantSet),PChar(SError),mb_IconError);
  end;
end;

procedure TPropEdit.UpdatePropertyValue;
begin
  SetCompPropValue;
end;

procedure TPropEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = vk_Return then
  begin
    if not FPostponeUpdate then SetCompPropValue;
  end;
  inherited;
end;

procedure TPropEdit.DoExit;
begin
  inherited;
  if FPostponeUpdate then exit;
  SetCompPropValue;
end;


// TPropRadioButton

procedure TPropRadioButton.Loaded;
begin
  inherited;
  GetCompPropValue;  
end;

procedure TPropRadioButton.SetPropertyName(Value: string);
begin
  if Value = FPropertyName then exit;

  FPropertyName := Value;

  if Value = '' then exit;

  GetCompPropValue;
end;

procedure TPropRadioButton.GetCompPropValue;
var
  i: Integer;
  S: String;
  BaseType: PPTypeInfo;
  PI: PPropInfo;
  TD: PTypeData;
  PV: TPropValue;
begin
  if (csLoading in ComponentState) or
   (csDesigning in ComponentState) then exit;

  Enabled := False;

  if FPropertyName = '' then exit;

  try
    PV := MacrosHandler.GetPropValueByMacros(FPropertyName,PR);

    with PV do
    begin
      with (PPropInfo(PR.CommandPointer))^.PropType^^ do
      begin
        case Kind of
          tkEnumeration:
          begin
            Enabled := True;
            PI:= PPropInfo(PR.CommandPointer);
            TD:= GetTypeData(PI.PropType^);
            BaseType := TD^.BaseType;
            i:= Integer(PV.Addr^);
            S:= GetEnumName(BaseType^, i);
            if S = FEnumName then
              Checked:= True;
          end;
        end;
      end;
    end;
  except
    MessageBox(Self.Handle, PChar(SCantGet),PChar(SError),mb_IconError);
  end;
  FInitialize:= True;
end;

procedure TPropRadioButton.SetCompPropValue;
const
  BooleanChar: array[False..True] of Char = ('-','+');
begin
  if not FInitialize then exit; 
  if FPropertyName = '' then exit;
  try
    with (PPropInfo(PR.CommandPointer))^.PropType^^ do
    begin
      case Kind of
        tkEnumeration:
        begin
          MacrosHandler.ExecMacros(FPropertyName + '= ' + FEnumName);
        end;
      end;
    end;
  except
    MessageBox(Self.Handle, PChar(SCantSet),PChar(SError),mb_IconError);
  end;
end;

procedure TPropRadioButton.SetEnumName(Value: string);
begin
  if Value = FEnumName then exit;

  FEnumName := Trim(Value);

  GetCompPropValue;
end;

procedure TPropRadioButton.SetChecked(Value: Boolean);
begin
  inherited;

  if (csDesigning in ComponentState) then exit;

  if not FPostponeUpdate then
    SetCompPropValue;
end;

procedure TPropRadioButton.UpdatePropertyValue;
begin
  SetCompPropValue;
end;

end.
